#include "Initialization.h"

UINT	SCLK_Frequency = 0;
UINT	CCLK_Frequency = 0;
cPLL_Config	EZKIT_594C_118S = { 0, 5, 22, 0, 0, 0, 0, 0 };	
cPLL_Config	EZKIT_594C_99S = { 0, 6, 22, 0, 0, 0, 0, 0 };	


//--------------------------------------------------------------------------//
// Function:	Init_Flags													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures PF8 as input for edge sensitive	//
//				interrupt generation.										//
//				The switch connected to PF8 (SW7) can be used to change the	//
//				direction of the moving light.								//
//--------------------------------------------------------------------------//
void Init_Flags(void)
{
	*pFIO_INEN		= 0x0100;
	*pFIO_DIR		= 0x0000;
	*pFIO_EDGE		= 0x0100;
	*pFIO_MASKA_D	= 0x0100;
}


//--------------------------------------------------------------------------//
// Function:	Init_EBIU													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function initialises and enables the asynchronous		//
//				memory banks for the External Bus Interface Unit (EBIU), so	//
//				that access to Flash A is possible.							//
//--------------------------------------------------------------------------//

void Init_EBIU(void)
{
	*pEBIU_AMBCTL0	= 0x7bb07bb0;
	*pEBIU_AMBCTL1	= 0x7bb07bb0;
	*pEBIU_AMGCTL	= 0x000f;
}


//--------------------------------------------------------------------------//
// Function:	Init_Flash													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets the pin direction of Port B in Flash A	//
//				to output.													//
//				The LEDs on the ADSP-BF533 EZ-KIT are connected to Port B.	//
//--------------------------------------------------------------------------//
void Init_Flash(void)
{
	*pFlashA_PortB_Dir = 0x3f;
}


//------------------------------------------------------------------------------//
// Function: Set_PLL_DIV														//
//																				//
// Parameters: CSEL (Core Clock Divider Select bits), SSEL (System Select 		//
//			   Divider Select Bits) 											//
//																				//
// Purpose: Configures the PLL_DIV register with the appropriate divider ratios //
//			to set the CCLK and SCLK to the desired values.                     //
//																				//			
// Returns: Will return "true" if user has entered divider values out of the    //
//		    acceptable range.										         	//
//------------------------------------------------------------------------------//
//#pragma optimize_for_speed
char Set_PLL_Div(unsigned short CSEL, unsigned short SSEL)
{
	if ((CSEL < 5) && (SSEL < 16))
	{
		*pPLL_DIV = ((CSEL << 4) | SSEL); 
		
		return 1;
	}
	
	else
		return 0; //error, Core Select or System Select values are out of range
		
}


//------------------------------------------------------------------------------//
// Function: Set_PLL_Frequencies														//
//																				//
// Parameters: CSEL (Core Clock Divider Select bits), SSEL (System Select 		//
//			   Divider Select Bits) 											//
//																				//
// Purpose: Configures the PLL_DIV register with the appropriate divider ratios //
//			to set the CCLK and SCLK to the desired values.                     //
//																				//			
// Returns: Will return "true" if user has entered divider values out of the    //
//		    acceptable range.										         	//
//------------------------------------------------------------------------------//
//#pragma optimize_off 
USHORT	Set_PLL_Frequencies( cPLL_Config * C )
{
	USHORT err;
	
	CCLK_Frequency = CLKIN_FREQUENCY*C->msel;
	SCLK_Frequency = CCLK_Frequency/C->ssel;
	
	err = Set_PLL_Div( C->csel, C->ssel );	
	err |= Set_PLL_CTL( C->msel, C->bypass, C->pwdn, C->stopck, C->pll_off, C->df );
	return err;
}


//------------------------------------------------------------------------------//
// Function: Set_PLL_CTL														//
//																				//
// Parameters: MSEL (VCO Multiplier Select bits), Bypass (PLL Bypass select),   //
//			   PDWN (Power Down), STOPCK (Stop Clock), PLL_OFF, DF (Divide      //
//			   Frequency).                                                      //
//																				//
// Purpose: Configures the fields in the PLL_CTL register. Note: changes will 	//
//			not take place until a specific PLL programing sequence is executed.//
//																				//			
// Returns: Will return 1 if any parameter values are outside the   			//
//			acceptable range.													//
//------------------------------------------------------------------------------//

char Set_PLL_CTL(USHORT msel, UCHAR bypass, UCHAR pwdn, UCHAR stopck, UCHAR pll_off, UCHAR df)
{
	if ((msel < 64) && (bypass < 2) && (pwdn < 2) && (stopck < 2) && (pll_off < 2) && (df < 2))
	{ 
		asm("CLI R2;"); /*disable interrupts, copy IMASK to R2 */
		
		*pPLL_CTL = ((msel << 9) | 	(bypass << 8) | (pwdn << 5) | (stopck << 3) | (pll_off << 1) | df);
		
		asm("IDLE;");   /*drain pipeline, enter idle state, wait for PLL wake-up */
		asm("STI R2;"); /*after PLL wakeup occurs, restore interrupts and IMASK */ 
		
		return 0;
	}
	
	else
	
		return 1; //error, one or more of the values is out of range
}

void Setup_Blackfin(void)
{
		
	sysreg_write(reg_SYSCFG, 0x32);		//Initialize System Configuration Register
	Set_PLL_Frequencies( &EZKIT_594C_118S);	

	Init_EBIU();

}
